import json
import pkg_resources

class RhymeChecker:
    def __init__(self, json_file_path=None):
        if json_file_path is None:
            json_file_path = pkg_resources.resource_filename(
                __name__, 'data/organized_ping_ze_rhyme_dict.json'
            )

        with open(json_file_path, 'r', encoding='utf-8') as file:
            self.ping_ze_dict = json.load(file)

        self.rhyme_dict = self._build_rhyme_dict()

    def _build_rhyme_dict(self):
        """
        Builds a dictionary where the keys are characters, and the values are lists of their rhyme groups.
        """
        rhyme_dict = {}
        for tone_type in self.ping_ze_dict:
            for tone_group in self.ping_ze_dict[tone_type]:
                for rhyme_category, characters in self.ping_ze_dict[tone_type][tone_group].items():
                    for char_group in characters:
                        for char in char_group:
                            rhyme_group = (tone_type, tone_group, rhyme_category)
                            if char in rhyme_dict:
                                rhyme_dict[char].append(rhyme_group)
                            else:
                                rhyme_dict[char] = [rhyme_group]
        return rhyme_dict
    
    def get_rhyme_group(self, char):
        """
        Returns the list of rhyme groups for a given character, or None if the character is not found.
        """
        return self.rhyme_dict.get(char)

    def do_rhyme(self, char1, char2):
        """
        Determines if two characters rhyme by comparing their rhyme groups and tone types.

        In the instance that a character has multiple rhyme groups due to multiple pronunciations, it returns true if it has one rhyme group in common with the other character.

        e.g. "鄉" is in 下平聲七陽 and 去聲二十三漾, it is assumed to use the rhyming pronunciation when compared to "昌" in 下平聲七陽
        """
        rhyme_groups1 = self.get_rhyme_group(char1)
        rhyme_groups2 = self.get_rhyme_group(char2)

        if not rhyme_groups1 or not rhyme_groups2:
            return False  # One or both characters are not in the rhyme data

        # Extract the core rhyme categories for each character (e.g., '上平聲一東' -> '一東')
        core_rhymes1 = {group[2].split('聲')[-1] for group in rhyme_groups1}
        core_rhymes2 = {group[2].split('聲')[-1] for group in rhyme_groups2}
        
        # Return True if there is any intersection between the two sets of core rhymes
        return not core_rhymes1.isdisjoint(core_rhymes2)
        
    def get_rhyme_type(self, char):
        """
        Returns the full list of rhyme types of the given character.
        """
        return self.get_rhyme_group(char)



import unittest
from pingshui_rhyme import PoemStructureChecker

class TestPoemStructure(unittest.TestCase):
    def setUp(self):
        self.checker = PoemStructureChecker()

    def test_jueju_rhyming(self):
        poem = '''
        床前明月光，
        疑是地上霜。
        舉頭望明月，
        低頭思故鄉。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Poem follows jueju rhyming rules.")

    def test_lushi_rhyming(self):
        poem = '''
        昔人已乘黃鶴去，
        此地空餘黃鶴樓。
        黃鶴一去不復返，
        白雲千載空悠悠。
        晴川歷歷漢陽樹，
        芳草萋萋鸚鵡洲。
        日暮鄉關何處是，
        煙波江上使人愁。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Poem follows lushi rhyming rules.")

    def test_pingze_meter(self):
        poem = '''
        歲莫陰陽催短景，天涯霜雪霽寒霄。
        五更鼓角聲悲壯，三峽星河影動搖。
        野哭千家聞戰伐，夷歌幾處起漁樵。
        臥龍躍馬終黃土，人事音書漫寂寥。 
        '''
        result, message = self.checker.check_poem_pingze_meter(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Poem follows the less restrictive ping-ze alternation pattern in 2nd, 4th, and 6th characters.")

    def test_messy_format(self):
        # A poem with tabs, weird spacing, and English punctuation
        poem = '''
        床前   明月光,
        疑是\t地上霜.
        舉頭   望明月;
        低頭\t思故鄉!
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Poem follows jueju rhyming rules.")

    def test_invalid_line_length(self):
        # A poem with 6 characters per line (invalid for jueju - must be 5 or 7)
        poem = '''
        床前看明月光，
        疑是看地上霜。
        舉頭看望明月，
        低頭看思故鄉。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertFalse(result)
        # With 4 lines, this is treated as jueju, which requires 5 or 7 chars per line
        self.assertIn("Each line must have 5 or 7 characters", message)

    def test_invalid_line_count(self):
        # A poem with only 3 lines (invalid)
        poem = '''
        床前明月光，
        疑是地上霜。
        舉頭望明月。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertFalse(result)
        self.assertIn("Poem must have at least 4 lines", message)

    def test_gushi_valid_5_char_10_lines(self):
        # A valid gushi with 10 lines, 5 characters each
        poem = '''
        明月何皎皎，照我羅牀幃
        憂愁不能寐，覽衣起徘徊
        客行雖云樂，不如早旋歸
        出戶獨彷徨，愁思當告誰
        引領還入房，淚下沾裳衣
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Potential Gushi (古詩) format detected - rhyme and meter checking not applicable.")

    def test_gushi_valid_7_char_10_lines(self):
        # A valid gushi with 10 lines, 7 characters each
        poem = '''
        昔人已乘黃鶴去，
        此地空餘黃鶴樓。
        黃鶴一去不復返，
        白雲千載空悠悠。
        晴川歷歷漢陽樹，
        芳草萋萋鸚鵡洲。
        日暮鄉關何處是，
        煙波江上使人愁。
        風起雲湧天地間，
        萬里江山入畫圖。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Potential Gushi (古詩) format detected - rhyme and meter checking not applicable.")

    def test_gushi_invalid_too_few_lines(self):
        # Invalid gushi - only 3 lines
        poem = '''
        白日依山盡，
        黃河入海流。
        欲窮千里目。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertFalse(result)
        self.assertEqual(message, "Poem must have at least 4 lines.")

    def test_gushi_invalid_inconsistent_length(self):
        # Invalid gushi - inconsistent line lengths
        poem = '''
        白日依山盡，
        黃河入海流。
        欲窮千里目去，
        更上一層樓。
        登高望遠處，
        天地共悠悠。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertFalse(result)
        self.assertEqual(message, "All lines in Gushi must have consistent character count.")

    def test_gushi_invalid_chars_too_short(self):
        # Invalid gushi - lines too short (3 chars)
        poem = '''
        白日山，
        黃河流。
        欲千里，
        更層樓。
        登望處，
        天悠悠。
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertFalse(result)
        self.assertEqual(message, "Each line in Gushi must have 4-8 characters.")

    def test_gushi_invalid_chars_too_long(self):
        # Invalid gushi - lines too long (9 chars)
        poem = '''
        白日依山盡黃河入海，
        黃河入海流欲窮千里，
        欲窮千里目更上一層，
        更上一層樓上一層樓，
        一層樓一層樓一層樓，
        '''
        result, message = self.checker.check_poem_rhyming(poem)
        self.assertFalse(result)
        self.assertEqual(message, "Each line in Gushi must have 4-8 characters.")

    def test_gushi_pingze_skipped(self):
        # Test that pingze meter check is skipped for gushi
        poem = '''
        青青河畔草，鬱鬱園中柳
        盈盈樓上女，皎皎當窗牖
        娥娥紅粉妝，纖纖出素手
        昔為倡家女，今為蕩子婦
        蕩子行不歸，空床難獨守
        '''
        result, message = self.checker.check_poem_pingze_meter(poem)
        self.assertTrue(result)
        self.assertEqual(message, "Potential Gushi (古詩) format detected - pingze meter checking not applicable.")

if __name__ == '__main__':
    unittest.main()


from .classifier import PingZeClassifier
from .rhymechecker import RhymeChecker
from .poem_structure_checker import PoemStructureChecker

__all__ = ['PingZeClassifier', 'RhymeChecker', 'PoemStructureChecker']